#include "caosdb/filestreaming/HandlerInterface.h"
#include "caosdb/entity/v1alpha1/main.grpc.pb.h"
#include "caosdb/entity.h"
#include "caosdb/filestreaming/FileWriter.h"

#include <grpcpp/grpcpp.h>

namespace FileExchange {
using caosdb::entity::FileDescriptor;
using caosdb::entity::v1alpha1::FileDownloadRequest;
using caosdb::entity::v1alpha1::FileDownloadResponse;
using caosdb::entity::v1alpha1::FileTransmissionService;

class DownloadRequestHandler : public HandlerInterface {
public:
  DownloadRequestHandler(HandlerTag tag, FileTransmissionService::Stub *stub,
                         grpc::CompletionQueue *cq,
                         const FileDescriptor &file_descriptor);

  ~DownloadRequestHandler() override = default;

  DownloadRequestHandler(const DownloadRequestHandler &) = delete;
  DownloadRequestHandler &operator=(const DownloadRequestHandler &) = delete;
  DownloadRequestHandler(DownloadRequestHandler &&) = delete;
  DownloadRequestHandler &operator=(DownloadRequestHandler &&) = delete;

  bool onNext(bool ok) override;

  void cancel() override;

protected:
  enum class CallState { NewCall, SendingRequest, ReceivingFile, CallComplete };

  void handleNewCallState();
  void handleSendingRequestState();
  void handleReceivingFileState();
  void handleCallCompleteState();

  HandlerTag tag_;

  FileTransmissionService::Stub *stub_;
  grpc::CompletionQueue *cq_;
  grpc::ClientContext ctx_;

  std::unique_ptr<grpc::ClientAsyncReader<FileDownloadResponse>> rpc_;

  FileDownloadRequest *request_;
  FileDownloadResponse *response_;
  grpc::Status status_;

  CallState state_;

  std::unique_ptr<FileWriter> fileWriter_;

  FileDescriptor file_descriptor_;

  unsigned long long bytesReceived_;
};

} // namespace FileExchange
