#pragma once

#include "caosdb/filestreaming/FileLock.h"

#include <boost/filesystem.hpp>
#include <boost/filesystem/fstream.hpp>
#include <boost/filesystem/string_file.hpp>
#include <fstream>
#include <memory>
#include <string>

namespace FileExchange {
using boost::filesystem::exists;
using boost::filesystem::ifstream;
using boost::filesystem::path;

class FileReader final {
public:
  FileReader(const boost::filesystem::path &filename);
  FileReader(const boost::filesystem::path &filename,
             const std::shared_ptr<FileMutex> &mutexPtr);

  ~FileReader() = default;

  FileReader(const FileReader &) = delete;
  FileReader &operator=(const FileReader &) = delete;

  FileReader(FileReader &&) = default;
  FileReader &operator=(FileReader &&) = default;

  unsigned long long fileSize() const { return size_; }

  std::size_t read(std::string &buffer);

private:
  void openFile();

  std::ifstream stream_;
  boost::filesystem::path filename_;
  unsigned long long size_;

  std::shared_ptr<FileMutex> mutexPtr_;
  FileReadLock lock_;
};

} // namespace FileExchange
