#include "caosdb/filestreaming/HandlerInterface.h"
#include "caosdb/entity.h"
#include "caosdb/entity/v1alpha1/main.grpc.pb.h"
#include "caosdb/filestreaming/FileManager.h"

#include <grpcpp/grpcpp.h>

namespace FileExchange {
using caosdb::entity::FileDescriptor;
using caosdb::entity::v1alpha1::FileTransmissionService;
using caosdb::entity::v1alpha1::FileUploadRequest;
using caosdb::entity::v1alpha1::FileUploadResponse;

class UploadRequestHandler : public HandlerInterface {
public:
  UploadRequestHandler(HandlerTag tag, FileTransmissionService::Stub *stub,
                       grpc::CompletionQueue *cq,
                       const FileDescriptor &file_descriptor);

  ~UploadRequestHandler() override = default;

  UploadRequestHandler(const UploadRequestHandler &) = delete;
  UploadRequestHandler &operator=(const UploadRequestHandler &) = delete;
  UploadRequestHandler(UploadRequestHandler &&) = delete;
  UploadRequestHandler &operator=(UploadRequestHandler &&) = delete;

  bool onNext(bool ok) override;

  void cancel() override;

protected:
  enum class CallState {
    NewCall,
    SendingHeader,
    SendingFile,
    ExpectingResponse,
    CallComplete
  };

  void handleNewCallState();
  void handleSendingHeaderState();
  void handleSendingFileState();
  void handleExpectingResponseState();
  void handleCallCompleteState();

  HandlerTag tag_;

  FileTransmissionService::Stub *stub_;
  grpc::CompletionQueue *cq_;
  grpc::ClientContext ctx_;

  std::unique_ptr<grpc::ClientAsyncWriter<FileUploadRequest>> rpc_;

  FileUploadRequest *request_;
  FileUploadResponse *response_;
  grpc::Status status_;

  CallState state_;

  std::unique_ptr<FileReader> fileReader_;

  FileDescriptor file_descriptor_;

  unsigned long long bytesToSend_;
};

} // namespace FileExchange
