#include "caosdb/filestreaming/Client.h"
#include "caosdb/logging.h"
#include "caosdb/status_code.h"
#include "caosdb/filestreaming/DownloadRequestHandler.h"
#include "caosdb/filestreaming/UploadRequestHandler.h"

namespace FileExchange {
using caosdb::StatusCode;

FileExchangeClient::~FileExchangeClient() {
  this->cancel();

  cq_.Shutdown();

  // drain the queue
  void *ignoredTag = nullptr;
  bool ok = false;
  while (cq_.Next(&ignoredTag, &ok))
    ;
}

StatusCode FileExchangeClient::upload(const FileDescriptor &file_descriptor) {
  handler_ = std::make_unique<UploadRequestHandler>(&handler_, stub_.get(),
                                                    &cq_, file_descriptor);

  int status = this->processMessages();
  if (status > 0) {
    return StatusCode::FILE_UPLOAD_ERROR;
  }
  return StatusCode::SUCCESS;
}

StatusCode FileExchangeClient::download(const FileDescriptor &file_descriptor) {
  handler_ = std::make_unique<DownloadRequestHandler>(&handler_, stub_.get(),
                                                      &cq_, file_descriptor);

  int status = this->processMessages();
  if (status > 0) {
    return StatusCode::FILE_DOWNLOAD_ERROR;
  }
  return StatusCode::SUCCESS;
}

void FileExchangeClient::cancel() {
  if (handler_) {
    handler_->cancel();
  }
}

int FileExchangeClient::processMessages() {
  try {
    void *tag = nullptr;
    bool ok = false;
    while (true) {
      if (cq_.Next(&tag, &ok)) {
        if (tag) {
          // TODO assert
          auto res = handler_->onNext(ok);
          if (!res) {
            // TODO comment
            handler_.reset();
            break;
          }
        } else {
          CAOSDB_LOG_ERROR(logger_name)
            << "Invalid tag delivered by notification queue.";
        }
      } else {
        CAOSDB_LOG_ERROR(logger_name)
          << "Notification queue has been shut down unexpectedly.";
        return 1;
      }
    }
  } catch (std::exception &e) {
    CAOSDB_LOG_ERROR(logger_name) << "Caught exception: " << e.what();
    return 1;
  } catch (...) {
    CAOSDB_LOG_ERROR(logger_name) << "Caught unknown exception.";
    return 1;
  }
  return 0;
}

} // namespace FileExchange
